<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model;

/**
 * Class Config
 * @package B2W\SkyHub\Model
 */
class Config
{
    /**
     * @var array
     */
    static protected $_config = array();

    /**
     * @return Config|bool
     */
    static public function instantiate()
    {
        static $instance = false;

        if ($instance === false) {
            $instance = new static();
        }

        return $instance;
    }

    /**
     * Config constructor.
     */
    private function __construct()
    {
        return $this;
    }

    /**
     * @return $this
     */
    private function __clone()
    {
        return $this;
    }

    /**
     * @param $path
     * @param null $key
     * @return array|mixed|null
     */
    public static function config($path, $key = null)
    {
        if (!isset(static::$_config[$path])) {
            $dir    = str_replace('/', DS, $path);
            $file   = \App::getBaseDir() . DS . 'config' . DS . $dir . '.php';

            if (file_exists($file)) {
                $data = require($file);
                static::$_config[$path] = $data;
            }
        }

        //return if dont exists file
        if (!isset(static::$_config[$path]) || empty(static::$_config[$path])) {
            return array();
        }

        //if not key sent, return full result
        if (empty($key) && isset(static::$_config[$path])) {
            return static::$_config[$path];
        }

        $config     = static::$_config[$path];
        $parts      = explode('/', $key);
        $partsCount = count($parts);

        for ($i = 0; $i < $partsCount; $i ++) {
            if (!isset($config[$parts[$i]])) {
                return null;
            }

            $config = $config[$parts[$i]];
        }

        return $config;
    }
}
