<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model\Transformer\Category;

use B2W\SkyHub\Model\Resource\Collection;

/**
 * Class DbToEntity
 * @package B2W\SkyHub\Model\Transformer\Category
 */
class EntityToApi
{
    /**
     * Convert Data
     *
     * @param Collection $collection
     * @param \SkyHub\Api\EntityInterface\Catalog\Product $parentTranformer
     * @return null
     */
    public function convert(Collection $collection, $parentTranformer)
    {
        /** @var \SkyHub\Api\EntityInterface\Catalog\Product $productInterface */
        $productInterface   = $parentTranformer->getInterface();

        /** @var \B2W\SkyHub\Model\Entity\CategoryEntity $category */
        foreach ($collection as $category) {
            if ($category->getParent()) {
                $category = $this->prepareCategoryParent($category, $category->getParent());
            }

            $productInterface->addCategory(
                $category->getCode(),
                $category->getName()
            );
        }

        return null;
    }

    /**
     * @param \B2W\SkyHub\Model\Entity\CategoryEntity $category
     * @param \B2W\SkyHub\Model\Entity\CategoryEntity $parent
     *
     * @return \B2W\SkyHub\Model\Entity\CategoryEntity
     */
    protected function prepareCategoryParent($category, $parent)
    {
        $category->setCode($parent->getCode() . ' > ' . $category->getCode());
        $category->setName($parent->getName() . ' > ' . $category->getName());
        
        if ($parent->getParent()) {
            $category = $this->prepareCategoryParent($category, $parent->getParent());
        }
        return $category;
    }
}
