<?php
/**
 * BSeller - B2W Companhia Digital
 *
 * DISCLAIMER
 *
 * Do not edit this file if you want to update this module for future new versions.
 *
 * @copyright     Copyright (c) 2017 B2W Companhia Digital. (http://www.bseller.com.br/)
 * @author        Luiz Tucillo <luiz.tucillo@e-smart.com.br>
 */

namespace B2W\SkyHub\Model\Transformer\Product\VariationAttributes;

use B2W\SkyHub\Contract\Entity\Product\SpecificationEntityInterface;
use B2W\SkyHub\Model\Entity\Product\AttributeEntity;
use B2W\SkyHub\Model\Entity\Product\VariationEntity;
use B2W\SkyHub\Model\Entity\ProductEntity;
use B2W\SkyHub\Model\Resource\Collection;
use B2W\SkyHub\Model\Transformer\DbToEntityAbstract;

/**
 * Class DbToEntity
 * @package B2W\SkyHub\Model\Transformer\Product\VariationAttributes
 */
class DbToEntity extends DbToEntityAbstract
{
    /**
     * @return VariationEntity|null
     */
    protected function _getEntityInstance()
    {
        return new VariationEntity();
    }

    /** @var ProductEntity */
    protected $_parentEntity = null;

    /**
     * @return array|mixed
     */
    protected function _getAttributeMap()
    {
        return array();
    }

    /**
     * @return Collection|null
     * @throws \Exception
     */
    public function convert()
    {
        $collection = new Collection();
        $attrs      = array();

        /** @var VariationEntity $variation */
        foreach ($this->_parentEntity->getVariations() as $variation) {
            /** @var SpecificationEntityInterface $specification */
            foreach ($variation->getSpecifications() as $specification) {
                /** @var AttributeEntity $attr */
                $attr = $specification->getAttribute();
                if (!$specification->getOption()) {
                    $attrs[$attr] = $specification;
                    continue;
                }

                if (!isset($attrs[$attr->getCode()])) {
                    $attrs[$attr->getCode()] = $attr;
                }
            }
        }

        foreach ($attrs as $attr) {
            $collection->addItem($attr);
        }

        return $collection;
    }
}
